@@ -1,9 +1,8 @@
-# This file is part of the GNU Health GTK Client.  The COPYRIGHT file at the top level of
+# This file is part of GNU Health.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-from common import *
-from datetime_strftime import *
+from .common import *
 from .domain_inversion import domain_inversion, eval_domain, localize_domain, \
         merge, inverse_leaf, filter_leaf, prepare_reference_domain, \
         extract_reference_models, concat, simplify, unique_value
-from environment import EvalEnvironment
-import timedelta
+from .environment import EvalEnvironment
+from . import timedelta
