@@ -1,21 +1,24 @@
-# This file is part of the GNU Health GTK Client.  The COPYRIGHT file at the top level of
+# This file is part of GNU Health.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 
-import ConfigParser
-import gtk
-import gobject
+import configparser
 import os
 import gettext
 import threading
+import logging
+
+from gi.repository import GLib, GObject, Gtk
 
 from tryton import __version__, SERVER_VERSION
+
 import tryton.common as common
 from tryton.config import CONFIG, GNUHEALTH_ICON, BANNER, \
     PIXMAPS_DIR, get_config_dir
 import tryton.rpc as rpc
-from tryton.exceptions import TrytonError
+from tryton.common.underline import set_underline
 
 _ = gettext.gettext
+logger = logging.getLogger(__name__)
 
 
 class DBListEditor(object):
@@ -29,105 +32,108 @@ class DBListEditor(object):
 
         # GTK Stuffs
         self.parent = parent
-        self.dialog = gtk.Dialog(title=_(u'Profile Editor'), parent=parent,
-            flags=gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT)
-        self.ok_button = self.dialog.add_button(gtk.STOCK_OK,
-            gtk.RESPONSE_ACCEPT)
-        self.dialog.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
+        self.dialog = Gtk.Dialog(
+            title=_('Profile Editor'), transient_for=parent,
+            modal=True, destroy_with_parent=True)
+        self.ok_button = self.dialog.add_button(
+            set_underline(_("Close")), Gtk.ResponseType.CLOSE)
+        self.dialog.set_position(Gtk.WindowPosition.CENTER_ON_PARENT)
         self.dialog.set_icon(GNUHEALTH_ICON)
 
-        hpaned = gtk.HPaned()
-        vbox_profiles = gtk.VBox(homogeneous=False, spacing=6)
-        self.cell = gtk.CellRendererText()
+        tooltips = common.Tooltips()
+
+        hpaned = Gtk.HPaned()
+        vbox_profiles = Gtk.VBox(homogeneous=False, spacing=6)
+        self.cell = Gtk.CellRendererText()
         self.cell.set_property('editable', True)
         self.cell.connect('editing-started', self.edit_started)
-        self.profile_tree = gtk.TreeView()
+        self.profile_tree = Gtk.TreeView()
         self.profile_tree.set_model(profile_store)
-        self.profile_tree.insert_column_with_attributes(-1, _(u'Profile'),
+        self.profile_tree.insert_column_with_attributes(-1, _('Profile'),
             self.cell, text=0)
         self.profile_tree.connect('cursor-changed', self.profile_selected)
-        scroll = gtk.ScrolledWindow()
-        scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
+        scroll = Gtk.ScrolledWindow()
+        scroll.set_policy(
+            Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
         scroll.add(self.profile_tree)
-        self.add_button = gtk.Button(_(u'_Add'), use_underline=True)
+        self.add_button = Gtk.Button()
+        self.add_button.set_image(common.IconFactory.get_image(
+                'tryton-add', Gtk.IconSize.BUTTON))
+        tooltips.set_tip(self.add_button, _("Add new profile"))
         self.add_button.connect('clicked', self.profile_create)
-        add_image = gtk.Image()
-        add_image.set_from_stock('gtk-add', gtk.ICON_SIZE_BUTTON)
-        self.add_button.set_image(add_image)
-        self.remove_button = gtk.Button(_(u'_Remove'), use_underline=True)
+        self.remove_button = Gtk.Button()
+        self.remove_button.set_image(common.IconFactory.get_image(
+                'tryton-remove', Gtk.IconSize.BUTTON))
+        tooltips.set_tip(self.remove_button, _("Remove selected profile"))
+        self.remove_button.get_style_context().add_class(
+            Gtk.STYLE_CLASS_DESTRUCTIVE_ACTION)
         self.remove_button.connect('clicked', self.profile_delete)
-        remove_image = gtk.Image()
-        remove_image.set_from_stock('gtk-remove', gtk.ICON_SIZE_BUTTON)
-        self.remove_button.set_image(remove_image)
-        vbox_profiles.pack_start(scroll, expand=True, fill=True)
-        vbox_profiles.pack_start(self.add_button, expand=False, fill=True)
-        vbox_profiles.pack_start(self.remove_button, expand=False, fill=True)
+        bbox = Gtk.ButtonBox()
+        bbox.pack_start(self.remove_button, expand=True, fill=True, padding=0)
+        bbox.pack_start(self.add_button, expand=True, fill=True, padding=0)
+        vbox_profiles.pack_start(scroll, expand=True, fill=True, padding=0)
+        vbox_profiles.pack_start(bbox, expand=False, fill=True, padding=0)
         hpaned.add1(vbox_profiles)
 
-        table = gtk.Table(4, 2, homogeneous=False)
-        table.set_row_spacings(3)
-        table.set_col_spacings(3)
-        host = gtk.Label(_(u'Host:'))
-        host.set_alignment(1, 0.5)
-        host.set_padding(3, 3)
-        self.host_entry = gtk.Entry()
+        grid = Gtk.Grid(column_spacing=3, row_spacing=3)
+        host = Gtk.Label(
+            label=set_underline(_('Host:')),
+            use_underline=True, halign=Gtk.Align.END)
+        self.host_entry = Gtk.Entry(hexpand=True)
         self.host_entry.connect('focus-out-event', self.display_dbwidget)
         self.host_entry.connect('changed', self.update_profiles, 'host')
         self.host_entry.set_activates_default(True)
         host.set_mnemonic_widget(self.host_entry)
-        table.attach(host, 0, 1, 1, 2, yoptions=False, xoptions=gtk.FILL)
-        table.attach(self.host_entry, 1, 2, 1, 2, yoptions=False)
-        database = gtk.Label(_(u'Database:'))
-        database.set_alignment(1, 0.5)
-        database.set_padding(3, 3)
-        self.database_entry = gtk.Entry()
+        grid.attach(host, 0, 1, 1, 1)
+        grid.attach(self.host_entry, 1, 1, 1, 1)
+        database = Gtk.Label(
+            label=set_underline(_('Database:')),
+            use_underline=True, halign=Gtk.Align.END)
+        self.database_entry = Gtk.Entry()
         self.database_entry.connect('changed', self.dbentry_changed)
         self.database_entry.connect('changed', self.update_profiles,
             'database')
         self.database_entry.set_activates_default(True)
-        self.database_label = gtk.Label()
+        self.database_label = Gtk.Label(valign=Gtk.Align.START)
         self.database_label.set_use_markup(True)
-        self.database_label.set_alignment(0, 0.5)
-        self.database_combo = gtk.ComboBox()
-        dbstore = gtk.ListStore(gobject.TYPE_STRING)
-        cell = gtk.CellRendererText()
-        self.database_combo.pack_start(cell, True)
-        self.database_combo.add_attribute(cell, 'text', 0)
-        self.database_combo.set_model(dbstore)
+        self.database_combo = Gtk.ComboBoxText()
         self.database_combo.connect('changed', self.dbcombo_changed)
-        self.database_progressbar = gtk.ProgressBar()
-        self.database_progressbar.set_text(_(u'Fetching databases list'))
-        image = gtk.Image()
-        image.set_from_stock('tryton-new', gtk.ICON_SIZE_BUTTON)
-        db_box = gtk.VBox(homogeneous=True)
-        db_box.pack_start(self.database_entry)
-        db_box.pack_start(self.database_combo)
-        db_box.pack_start(self.database_label)
-        db_box.pack_start(self.database_progressbar)
+        self.database_progressbar = Gtk.ProgressBar()
+        self.database_progressbar.set_text(_('Fetching databases list'))
+        db_box = Gtk.VBox(homogeneous=True, hexpand=True)
+        db_box.pack_start(
+            self.database_entry, expand=True, fill=True, padding=0)
+        db_box.pack_start(
+            self.database_combo, expand=True, fill=True, padding=0)
+        db_box.pack_start(
+            self.database_label, expand=True, fill=True, padding=0)
+        db_box.pack_start(
+            self.database_progressbar, expand=True, fill=True, padding=0)
         # Compute size_request of box in order to prevent "form jumping"
         width, height = 0, 0
         for child in db_box.get_children():
-            cwidth, cheight = child.size_request()
-            width, height = max(width, cwidth), max(height, cheight)
+            request = child.get_preferred_size()[0]
+            width = max(width, request.width)
+            height = max(height, request.height)
         db_box.set_size_request(width, height)
-        table.attach(database, 0, 1, 2, 3, yoptions=False, xoptions=gtk.FILL)
-        table.attach(db_box, 1, 2, 2, 3, yoptions=False)
-        username = gtk.Label(_(u'Username:'))
-        username.set_alignment(1, 0.5)
-        username.set_padding(3, 3)
-        self.username_entry = gtk.Entry()
+        grid.attach(database, 0, 2, 1, 1)
+        grid.attach(db_box, 1, 2, 1, 1)
+        username = Gtk.Label(
+            label=set_underline(_('Username:')),
+            use_underline=True, halign=Gtk.Align.END)
+        self.username_entry = Gtk.Entry(hexpand=True)
         self.username_entry.connect('changed', self.update_profiles,
             'username')
         username.set_mnemonic_widget(self.username_entry)
         self.username_entry.set_activates_default(True)
-        table.attach(username, 0, 1, 3, 4, yoptions=False, xoptions=gtk.FILL)
-        table.attach(self.username_entry, 1, 2, 3, 4, yoptions=False)
-        hpaned.add2(table)
+        grid.attach(username, 0, 3, 1, 1)
+        grid.attach(self.username_entry, 1, 3, 1, 1)
+        hpaned.add2(grid)
         hpaned.set_position(250)
 
-        self.dialog.vbox.pack_start(hpaned)
+        self.dialog.vbox.pack_start(hpaned, expand=True, fill=True, padding=0)
         self.dialog.set_default_size(640, 350)
-        self.dialog.set_default_response(gtk.RESPONSE_ACCEPT)
+        self.dialog.set_default_response(Gtk.ResponseType.CLOSE)
 
         self.dialog.connect('close', lambda *a: False)
         self.dialog.connect('response', self.response)
@@ -209,7 +215,7 @@ class DBListEditor(object):
             try:
                 entry_value = self.profiles.get(self.current_profile['name'],
                     field)
-            except ConfigParser.NoOptionError:
+            except configparser.NoOptionError:
                 entry_value = ''
             entry.set_text(entry_value)
             if field == 'database':
@@ -218,7 +224,7 @@ class DBListEditor(object):
         self.display_dbwidget(None, None)
 
     def edit_started(self, renderer, editable, path):
-        if isinstance(editable, gtk.Entry):
+        if isinstance(editable, Gtk.Entry):
             editable.connect('focus-out-event', self.edit_profilename,
                 renderer, path)
 
@@ -281,7 +287,7 @@ class DBListEditor(object):
         self.dbs_updated = threading.Event()
         threading.Thread(target=self.refresh_databases_start,
             args=(host, port)).start()
-        gobject.timeout_add(100, self.refresh_databases_end, host, port)
+        GLib.timeout_add(100, self.refresh_databases_end, host, port)
 
     def refresh_databases_start(self, host, port):
         dbs = None
@@ -303,18 +309,17 @@ class DBListEditor(object):
 
         label = None
         if self.test_server_version(host, port) is False:
-            label = _(u'Incompatible version of the server')
+            label = _('Incompatible version of the server')
         elif dbs is None:
-            label = _(u'Could not connect to the server')
+            label = _('Could not connect to the server')
         if label:
             self.database_label.set_label('<b>%s</b>' % label)
             self.database_label.show()
         else:
-            liststore = self.database_combo.get_model()
-            liststore.clear()
+            self.database_combo.remove_all()
             index = -1
             for db_num, db_name in enumerate(dbs):
-                liststore.append([db_name])
+                self.database_combo.append_text(db_name)
                 if self.current_database and db_name == self.current_database:
                     index = db_num
             if index == -1:
@@ -377,122 +382,121 @@ class DBListEditor(object):
         try:
             int(new_value)
         except ValueError:
-            entry.stop_emission('insert-text')
+            entry.stop_emission_by_name('insert-text')
 
 
 class DBLogin(object):
     def __init__(self):
-        # GTK Stuffs
-        self.parent = common.get_toplevel_window()
-        self.dialog = gtk.Dialog(title=_('Login'), parent=self.parent,
-            flags=gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT)
-        self.dialog.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
+        # Fake windows to avoid warning about Dialog without transient
+        self._window = Gtk.Window()
+        self.dialog = Gtk.Dialog(title="GNU Health HMIS - " + _('Login'), modal=True)
+        self.dialog.set_transient_for(self._window)
         self.dialog.set_icon(GNUHEALTH_ICON)
+        self.dialog.set_position(Gtk.WindowPosition.CENTER_ALWAYS)
+        self.dialog.set_resizable(False)
 
         tooltips = common.Tooltips()
-        button_cancel = gtk.Button(_('_Cancel'), use_underline=True)
-        img_cancel = gtk.Image()
-        img_cancel.set_from_stock('gtk-cancel', gtk.ICON_SIZE_BUTTON)
-        button_cancel.set_image(img_cancel)
+        button_cancel = Gtk.Button(label=_('_Cancel'), use_underline=True)
         tooltips.set_tip(button_cancel,
-            _('Cancel connection to the Tryton server'))
-        self.dialog.add_action_widget(button_cancel, gtk.RESPONSE_CANCEL)
-        self.button_connect = gtk.Button(_('C_onnect'), use_underline=True)
-        img_connect = gtk.Image()
-        img_connect.set_from_stock('tryton-connect', gtk.ICON_SIZE_BUTTON)
-        self.button_connect.set_image(img_connect)
+            _('Cancel connection to the GNU HEALTH HMIS server'))
+        self.dialog.add_action_widget(button_cancel, Gtk.ResponseType.CANCEL)
+        self.button_connect = Gtk.Button(
+            label=_('C_onnect'), use_underline=True)
+        self.button_connect.get_style_context().add_class(
+            Gtk.STYLE_CLASS_SUGGESTED_ACTION)
         self.button_connect.set_can_default(True)
-        tooltips.set_tip(self.button_connect, _('Connect the Tryton server'))
-        self.dialog.add_action_widget(self.button_connect, gtk.RESPONSE_OK)
-        self.dialog.set_default_response(gtk.RESPONSE_OK)
-        alignment = gtk.Alignment(yalign=0, yscale=0, xscale=1)
-        self.table_main = gtk.Table(3, 3, False)
-        self.table_main.set_border_width(0)
-        self.table_main.set_row_spacings(3)
-        self.table_main.set_col_spacings(3)
-        alignment.add(self.table_main)
-        self.dialog.vbox.pack_start(alignment, True, True, 0)
+        tooltips.set_tip(self.button_connect,
+                         _('Connect the GNU Health HMIS server'))
+        self.dialog.add_action_widget(self.button_connect, Gtk.ResponseType.OK)
+        self.dialog.set_default_response(Gtk.ResponseType.OK)
+        alignment = Gtk.Alignment(yalign=0, yscale=0, xscale=1)
+        grid = Gtk.Grid(column_spacing=3, row_spacing=3)
+        alignment.add(grid)
+        self.dialog.vbox.pack_start(
+            alignment, expand=True, fill=True, padding=0)
 
-        image = gtk.Image()
+        image = Gtk.Image()
         image.set_from_file(os.path.join(PIXMAPS_DIR, BANNER))
-        image.set_alignment(0.5, 1)
-        ebox = gtk.EventBox()
-        ebox.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse("#1b2019"))
-        ebox.add(image)
-        self.table_main.attach(ebox, 0, 3, 0, 1, ypadding=2)
+        image.set_valign(Gtk.Align.START)
+        overlay = Gtk.Overlay()
+        overlay.add(image)
+        label = Gtk.Label(
+            label='<span color="white">%s</span>' % __version__,
+            use_markup=True)
+        label.props.halign = Gtk.Align.END
+        label.props.valign = Gtk.Align.START
+        label.props.margin_right = 10
+        label.props.margin_top = 5
+        overlay.add_overlay(label)
+        grid.attach(overlay, 0, 0, 3, 1)
 
-        self.profile_store = gtk.ListStore(gobject.TYPE_STRING,
-            gobject.TYPE_BOOLEAN)
-        self.combo_profile = gtk.ComboBox()
-        cell = gtk.CellRendererText()
-        self.combo_profile.pack_start(cell, True)
+        self.profile_store = Gtk.ListStore(
+            GObject.TYPE_STRING, GObject.TYPE_BOOLEAN)
+        self.combo_profile = Gtk.ComboBox(hexpand=True)
+        cell = Gtk.CellRendererText()
+        self.combo_profile.pack_start(cell, expand=True)
         self.combo_profile.add_attribute(cell, 'text', 0)
         self.combo_profile.add_attribute(cell, 'sensitive', 1)
         self.combo_profile.set_model(self.profile_store)
         self.combo_profile.connect('changed', self.profile_changed)
-        self.profile_label = gtk.Label(_(u'Profile:'))
-        self.profile_label.set_justify(gtk.JUSTIFY_RIGHT)
-        self.profile_label.set_alignment(1, 0.5)
-        self.profile_label.set_padding(3, 3)
-        self.profile_button = gtk.Button(_('_Manage profiles'),
-            use_underline=True)
+        self.profile_label = Gtk.Label(
+            label=set_underline(_('Profile:')),
+            use_underline=True, halign=Gtk.Align.END)
+        self.profile_label.set_mnemonic_widget(self.combo_profile)
+        self.profile_button = Gtk.Button(
+            label=set_underline(_('Manage...')), use_underline=True)
         self.profile_button.connect('clicked', self.profile_manage)
-        self.table_main.attach(self.profile_label, 0, 1, 1, 2,
-            xoptions=gtk.FILL)
-        self.table_main.attach(self.combo_profile, 1, 2, 1, 2)
-        self.table_main.attach(self.profile_button, 2, 3, 1, 2,
-            xoptions=gtk.FILL)
-        image = gtk.Image()
-        image.set_from_stock('gtk-edit', gtk.ICON_SIZE_BUTTON)
-        self.profile_button.set_image(image)
-        self.expander = gtk.Expander()
+        grid.attach(self.profile_label, 0, 1, 1, 1)
+        grid.attach(self.combo_profile, 1, 1, 1, 1)
+        grid.attach(self.profile_button, 2, 1, 1, 1)
+        self.expander = Gtk.Expander()
         self.expander.set_label(_('Host / Database information'))
         self.expander.connect('notify::expanded', self.expand_hostspec)
-        self.table_main.attach(self.expander, 0, 3, 3, 4)
-        self.label_host = gtk.Label(_('Host:'))
-        self.label_host.set_justify(gtk.JUSTIFY_RIGHT)
-        self.label_host.set_alignment(1, 0.5)
-        self.label_host.set_padding(3, 3)
-        self.entry_host = gtk.Entry()
+        grid.attach(self.expander, 0, 2, 3, 1)
+        self.label_host = Gtk.Label(
+            label=set_underline(_('Host:')),
+            use_underline=True, halign=Gtk.Align.END)
+        self.entry_host = Gtk.Entry(hexpand=True)
         self.entry_host.connect_after('focus-out-event',
             self.clear_profile_combo)
         self.entry_host.set_activates_default(True)
         self.label_host.set_mnemonic_widget(self.entry_host)
-        self.table_main.attach(self.label_host, 0, 1, 4, 5, xoptions=gtk.FILL)
-        self.table_main.attach(self.entry_host, 1, 3, 4, 5)
-        self.label_database = gtk.Label(_('Database:'))
-        self.label_database.set_justify(gtk.JUSTIFY_RIGHT)
-        self.label_database.set_alignment(1, 0.5)
-        self.label_database.set_padding(3, 3)
-        self.entry_database = gtk.Entry()
+        grid.attach(self.label_host, 0, 3, 1, 1)
+        grid.attach(self.entry_host, 1, 3, 2, 1)
+        self.label_database = Gtk.Label(
+            label=set_underline(_('Database:')),
+            use_underline=True, halign=Gtk.Align.END)
+        self.entry_database = Gtk.Entry(hexpand=True)
         self.entry_database.connect_after('focus-out-event',
             self.clear_profile_combo)
         self.entry_database.set_activates_default(True)
         self.label_database.set_mnemonic_widget(self.entry_database)
-        self.table_main.attach(self.label_database, 0, 1, 5, 6,
-            xoptions=gtk.FILL)
-        self.table_main.attach(self.entry_database, 1, 3, 5, 6)
-        self.entry_login = gtk.Entry()
+        grid.attach(self.label_database, 0, 4, 1, 1)
+        grid.attach(self.entry_database, 1, 4, 2, 1)
+        self.entry_login = Gtk.Entry(hexpand=True)
         self.entry_login.set_activates_default(True)
-        self.table_main.attach(self.entry_login, 1, 3, 6, 7)
-        label_username = gtk.Label(_("User name:"))
-        label_username.set_alignment(1, 0.5)
-        label_username.set_padding(3, 3)
+        grid.attach(self.entry_login, 1, 5, 2, 1)
+        label_username = Gtk.Label(
+            label=set_underline(_("User name:")),
+            use_underline=True, halign=Gtk.Align.END, margin=3)
         label_username.set_mnemonic_widget(self.entry_login)
-        self.table_main.attach(label_username, 0, 1, 6, 7, xoptions=gtk.FILL)
+        grid.attach(label_username, 0, 5, 1, 1)
 
-        # Profile informations
+        # Profile information
         self.profile_cfg = os.path.join(get_config_dir(), 'profiles.cfg')
-        self.profiles = ConfigParser.SafeConfigParser()
+        self.profiles = configparser.ConfigParser()
         if not os.path.exists(self.profile_cfg):
             short_version = '.'.join(__version__.split('.', 2)[:2])
-            name = 'health.gnusolidario.org'
+            name = 'federation.gnuhealth.org'
             self.profiles.add_section(name)
             self.profiles.set(name, 'host', name)
             self.profiles.set(name, 'database', 'health%s' % short_version)
             self.profiles.set(name, 'username', 'admin')
         else:
-            self.profiles.read(self.profile_cfg)
+            try:
+                self.profiles.read(self.profile_cfg)
+            except configparser.ParsingError:
+                logger.error("Fail to parse profiles.cfg", exc_info=True)
         for section in self.profiles.sections():
             active = all(self.profiles.has_option(section, option)
                 for option in ('host', 'database'))
@@ -500,7 +504,7 @@ class DBLogin(object):
 
     def profile_manage(self, widget):
         def callback(profile_name):
-            with open(self.profile_cfg, 'wb') as configfile:
+            with open(self.profile_cfg, 'w') as configfile:
                 self.profiles.write(configfile)
 
             for idx, row in enumerate(self.profile_store):
@@ -524,7 +528,7 @@ class DBLogin(object):
         profile = self.profile_store[position][0]
         try:
             username = self.profiles.get(profile, 'username')
-        except ConfigParser.NoOptionError:
+        except configparser.NoOptionError:
             username = ''
         host = self.profiles.get(profile, 'host')
         self.entry_host.set_text('%s' % host)
@@ -544,9 +548,12 @@ class DBLogin(object):
             if not profile_info[1]:
                 continue
             profile = profile_info[0]
-            profile_host = self.profiles.get(profile, 'host')
-            profile_db = self.profiles.get(profile, 'database')
-            profile_login = self.profiles.get(profile, 'username')
+            try:
+                profile_host = self.profiles.get(profile, 'host')
+                profile_db = self.profiles.get(profile, 'database')
+                profile_login = self.profiles.get(profile, 'username')
+            except configparser.NoOptionError:
+                continue
             if (host == common.get_hostname(profile_host)
                     and port == common.get_port(profile_host)
                     and database == profile_db
@@ -568,12 +575,15 @@ class DBLogin(object):
         profile_name = CONFIG['login.profile']
         can_use_profile = self.profiles.has_section(profile_name)
         if can_use_profile:
-            for (configname, sectionname) in [
+            for (configname, option) in [
                     ('login.host', 'host'),
                     ('login.db', 'database'),
                     ]:
-                if (self.profiles.get(profile_name, sectionname)
-                        != CONFIG[configname]):
+                try:
+                    value = self.profiles.get(profile_name, option)
+                except configparser.NoOptionError:
+                    value = None
+                if value != CONFIG[configname]:
                     can_use_profile = False
                     break
 
@@ -594,8 +604,6 @@ class DBLogin(object):
 
         self.entry_login.grab_focus()
 
-        # Reshow dialog for gtk-quarks
-        self.dialog.reshow_with_initial_size()
         self.expander.set_expanded(CONFIG['login.expanded'])
         # The previous action did not called expand_hostspec
         self.expand_hostspec(self.expander)
@@ -603,7 +611,7 @@ class DBLogin(object):
         response, result = None, ('', '', '', '')
         while not all(result):
             response = self.dialog.run()
-            if response != gtk.RESPONSE_OK:
+            if response != Gtk.ResponseType.OK:
                 break
             self.clear_profile_combo()
             active_profile = self.combo_profile.get_active()
@@ -618,10 +626,12 @@ class DBLogin(object):
             if not test:
                 if test is False:
                     common.warning('',
-                        _(u'Incompatible version of the server'))
+                        _('Incompatible version of the server'),
+                        parent=self.dialog)
                 else:
                     common.warning('',
-                        _(u'Could not connect to the server'))
+                        _('Could not connect to the server'),
+                        parent=self.dialog)
                 continue
             database = self.entry_database.get_text()
             login = self.entry_login.get_text()
@@ -633,9 +643,6 @@ class DBLogin(object):
             result = (
                 hostname, port, database, self.entry_login.get_text())
 
-        self.parent.present()
         self.dialog.destroy()
-        if response != gtk.RESPONSE_OK:
-            rpc.logout()
-            raise TrytonError('QueryCanceled')
-        return result
+        self._window.destroy()
+        return response == Gtk.ResponseType.OK
