@@ -1,19 +1,19 @@
-# This file is part of the GNU Health GTK Client.  The COPYRIGHT file at the top level of
+# This file is part of GNU Health.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 import calendar
 import datetime
 import goocalendar
-from dates_period import DatesPeriod
+from .dates_period import DatesPeriod
 
 
 class Calendar_(goocalendar.Calendar):
     'Calendar'
 
-    def __init__(self, attrs, screen, fields, event_store=None):
+    def __init__(self, attrs, view, fields, event_store=None):
         super(Calendar_, self).__init__(
             event_store, attrs.get('mode', 'month'))
         self.attrs = attrs
-        self.screen = screen
+        self.view_calendar = view
         self.fields = fields
         self.event_store = event_store
         self.current_domain_period = self.get_displayed_period()
@@ -22,9 +22,14 @@ class Calendar_(goocalendar.Calendar):
         dtstart = self.attrs['dtstart']
         record[dtstart].set(record, datetime.datetime.combine(selected_date,
             datetime.time(0)))
+        record.on_change([dtstart])
+        record.on_change_with([dtstart])
 
     def get_displayed_period(self):
         cal = calendar.Calendar(self.firstweekday)
+        if self.view == 'day':
+            first_date = self.selected_date
+            last_date = self.selected_date + datetime.timedelta(1)
         if self.view == 'week':
             week = goocalendar.util.my_weekdatescalendar(cal,
                 self.selected_date)
@@ -74,8 +79,8 @@ class Calendar_(goocalendar.Calendar):
     def display(self, group):
         dtstart = self.attrs['dtstart']
         dtend = self.attrs.get('dtend')
-        if self.screen.current_record:
-            record = self.screen.current_record
+        if self.view_calendar.record:
+            record = self.view_calendar.record
             date = record[dtstart].get(record)
             if date:  # select the day of the current record
                 self.select(date)
