@@ -1,4 +1,4 @@
-# This file is part of the GNU Health GTK Client.  The COPYRIGHT file at the top level of
+# This file is part of GNU Health.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 # This code is inspired by the pycha project
 # (http://www.lorenzogil.com/projects/pycha/)
@@ -8,7 +8,7 @@ import datetime
 
 import cairo
 
-from graph import Graph
+from .graph import Graph
 import tryton.common as common
 import tryton.rpc as rpc
 
@@ -23,7 +23,7 @@ class Line(Graph):
 
         self.points = []
         i = 0
-        keys = self.datas.keys()
+        keys = list(self.datas.keys())
         keys.sort()
         for xfield in keys:
             j = 0
