@@ -296,7 +296,7 @@ class TreeXMLViewParser(XMLViewParser):
         column.name = name
 
         prefixes = []
-        suffixes = []
+        suffixes = list(widget.suffixes)
         if attributes['widget'] in ['url', 'email', 'callto', 'sip']:
             prefixes.append(
                 Affix(self.view, attributes, protocol=attributes['widget']))
@@ -439,7 +439,6 @@ class ViewTree(View):
         self.children_field = children_field
         self.sum_widgets = []
         self.sum_box = Gtk.HBox()
-        self.reload = False
         self.treeview = None
         editable = xml.getAttribute('editable')
         if editable and not screen.readonly:
@@ -1000,10 +999,10 @@ class ViewTree(View):
     def reset(self):
         pass
 
-    def display(self):
+    def display(self, force=False):
         self.treeview.display_counter += 1
         current_record = self.record
-        if (self.reload
+        if (force
                 or not self.treeview.get_model()
                 or self.group != self.treeview.get_model().group):
             model = AdaptModelGroup(self.group, self.children_field)
@@ -1014,7 +1013,6 @@ class ViewTree(View):
                 selection = self.treeview.get_selection()
                 path = current_record.get_index_path(model.group)
                 selection.select_path(path)
-        self.reload = False
         if not current_record:
             selection = self.treeview.get_selection()
             selection.unselect_all()
