@@ -1,38 +1,46 @@
-# This file is part of the GNU Health GTK Client.  The COPYRIGHT file at the top level of
+# This file is part of GNU Health.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
 import datetime
-
-import gtk
 import gettext
-import gobject
+
+from gi.repository import Gdk, GLib, GObject, Gtk
 
 import tryton.common as common
 from tryton.common.domain_parser import quote
-from tryton.common.placeholder_entry import PlaceholderEntry
 from tryton.common.treeviewcontrol import TreeViewControl
 from tryton.common.datetime_ import Date, Time, DateTime, add_operators
+from tryton.common.number_entry import NumberEntry
 from tryton.pyson import PYSONDecoder
 
 _ = gettext.gettext
 
 
-class Dates(gtk.HBox):
+class Between(Gtk.HBox):
+    _changed_signal = None
 
-    def __init__(self, format_=None, _entry=Date):
-        super(Dates, self).__init__()
-        self.from_ = add_operators(_entry())
-        self.pack_start(self.from_, expand=True, fill=True)
-        self.pack_start(gtk.Label(_('..')), expand=False, fill=False)
-        self.to = add_operators(_entry())
-        self.pack_start(self.to, expand=True, fill=True)
-        if format_:
-            self.from_.props.format = format_
-            self.to.props.format = format_
+    def __init__(self, _entry=Gtk.Entry):
+        super().__init__()
+        self.from_ = _entry()
+        self.pack_start(self.from_, expand=True, fill=True, padding=0)
+        self.pack_start(
+            Gtk.Label(label=_('..')), expand=False, fill=False, padding=0)
+        self.to = _entry()
+        self.pack_start(self.to, expand=True, fill=True, padding=0)
 
-    def _get_value(self, widget):
-        value = widget.props.value
-        if value:
-            return common.datetime_strftime(value, widget.props.format)
+        if self._changed_signal:
+            self.from_.connect_after(
+                self._changed_signal, self._from_changed)
+
+    @property
+    def _connect_widgets(self):
+        return [self.from_, self.to]
+
+    def connect(self, signal, callback):
+        for widget in self._connect_widgets:
+            try:
+                widget.connect(signal, callback)
+            except TypeError:
+                pass
 
     def get_value(self):
         from_ = self._get_value(self.from_)
@@ -47,73 +55,123 @@ class Dates(gtk.HBox):
         elif to:
             return '<=%s' % quote(to)
 
-    def connect_activate(self, callback):
-        self.from_.connect('activate', callback)
-        self.to.connect('activate', callback)
+    def _get_value(self, entry):
+        raise NotImplementedError
 
-    def set_values(self, from_, to):
-        self.from_.props.value = from_
-        self.to.props.value = to
+    def set_value(self, from_, to):
+        self._set_value(self.from_, from_)
+        self._set_value(self.to, to)
+
+    def _set_value(self, entry, value):
+        raise NotImplementedError
+
+    def _from_changed(self, widget):
+        self._set_value(self.to, self._get_value(self.from_))
 
 
-class Times(Dates):
+class WithOperators:
+    def __init__(self, entry):
+        self.entry = entry
 
-    def __init__(self, format_, _entry=Time):
-        super(Times, self).__init__(_entry=_entry)
+    def __call__(self):
+        return add_operators(self.entry())
+
+
+class BetweenDates(Between):
+    _entry = None
+
+    def __init__(self, format_=None):
+        super().__init__(WithOperators(self._entry))
+        if format_:
+            self.from_.props.format = format_
+            self.to.props.format = format_
+
+    def _set_value(self, entry, value):
+        entry.props.value = value
+
+
+class Dates(BetweenDates):
+    _entry = Date
+    _changed_signal = 'date-changed'
+
+    def _get_value(self, widget):
+        value = widget.props.value
+        if value:
+            return value.strftime(widget.props.format)
+
+
+class Times(BetweenDates):
+    _entry = Time
+    _changed_signal = 'time-changed'
+
+    @property
+    def _connect_widgets(self):
+        return [self.from_.get_child(), self.to.get_child()]
 
     def _get_value(self, widget):
         value = widget.props.value
         if value:
             return datetime.time.strftime(value, widget.props.format)
 
-    def connect_activate(self, callback):
-        for widget in self.from_.get_children() + self.to.get_children():
-            widget.connect('activate', callback)
 
+class DateTimes(BetweenDates):
+    _entry = DateTime
+    _changed_signal = 'datetime-changed'
 
-class DateTimes(Dates):
-
-    def __init__(self, date_format, time_format, _entry=DateTime):
-        super(DateTimes, self).__init__(_entry=_entry)
+    def __init__(self, date_format, time_format):
+        super().__init__()
         self.from_.props.date_format = date_format
         self.to.props.date_format = date_format
         self.from_.props.time_format = time_format
         self.to.props.time_format = time_format
 
+    @property
+    def _connect_widgets(self):
+        return self.from_.get_children() + self.to.get_children()
+
     def _get_value(self, widget):
         value = widget.props.value
         if value:
-            return common.datetime_strftime(value,
+            return value.strftime(
                 widget.props.date_format + ' ' + widget.props.time_format)
 
-    def connect_activate(self, callback):
-        for widget in self.from_.get_children() + self.to.get_children():
-            if isinstance(widget, Date):
-                widget.connect('activate', callback)
-            elif isinstance(widget, Time):
-                widget.get_child().connect('activate', callback)
+
+class Numbers(Between):
+    _changed_signal = 'changed'
+
+    def __init__(self):
+        super().__init__(NumberEntry)
+
+    def _get_value(self, widget):
+        return widget.get_text()
+
+    def _set_value(self, entry, value):
+        entry.set_text(value or '')
 
 
-class Selection(gtk.ScrolledWindow):
+class Selection(Gtk.ScrolledWindow):
 
     def __init__(self, selections):
         super(Selection, self).__init__()
         self.treeview = TreeViewControl()
-        model = gtk.ListStore(gobject.TYPE_STRING)
+        model = Gtk.ListStore(GObject.TYPE_STRING)
         for selection in selections:
             model.append((selection,))
         self.treeview.set_model(model)
 
-        column = gtk.TreeViewColumn()
-        cell = gtk.CellRendererText()
-        column.pack_start(cell)
+        column = Gtk.TreeViewColumn()
+        cell = Gtk.CellRendererText()
+        column.pack_start(cell, expand=True)
         column.add_attribute(cell, 'text', 0)
         self.treeview.append_column(column)
-        self.treeview.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
+        self.treeview.get_selection().set_mode(Gtk.SelectionMode.MULTIPLE)
         self.treeview.set_headers_visible(False)
         self.add(self.treeview)
-        self.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
-        self.set_shadow_type(gtk.SHADOW_ETCHED_IN)
+        self.set_policy(
+            Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
+        self.set_shadow_type(Gtk.ShadowType.ETCHED_IN)
+        self.set_min_content_height(min(20 * len(selections), 200))
+        self.set_max_content_height(200)
 
     def get_value(self):
         values = []
@@ -129,33 +187,34 @@ class Selection(gtk.ScrolledWindow):
 class ScreenContainer(object):
 
     def __init__(self, tab_domain):
-        self.viewport = gtk.Viewport()
-        self.viewport.set_shadow_type(gtk.SHADOW_NONE)
-        self.vbox = gtk.VBox(spacing=3)
-        self.alternate_viewport = gtk.Viewport()
-        self.alternate_viewport.set_shadow_type(gtk.SHADOW_NONE)
+        self.viewport = Gtk.Viewport()
+        self.viewport.set_shadow_type(Gtk.ShadowType.NONE)
+        self.vbox = Gtk.VBox(spacing=3)
+        self.alternate_viewport = Gtk.Viewport()
+        self.alternate_viewport.set_shadow_type(Gtk.ShadowType.NONE)
         self.alternate_view = False
-        self.search_window = None
-        self.search_table = None
+        self.search_popover = None
+        self.search_grid = None
         self.last_search_text = ''
         self.tab_domain = tab_domain or []
         self.tab_counter = []
 
         tooltips = common.Tooltips()
 
-        self.filter_vbox = gtk.VBox(spacing=0)
+        self.filter_vbox = Gtk.VBox(spacing=0)
         self.filter_vbox.set_border_width(0)
-        hbox = gtk.HBox(homogeneous=False, spacing=0)
+        hbox = Gtk.HBox(homogeneous=False, spacing=0)
 
-        self.search_entry = PlaceholderEntry()
+        self.search_entry = Gtk.Entry()
         self.search_entry.set_placeholder_text(_('Search'))
         self.search_entry.set_alignment(0.0)
-        self.search_entry.set_icon_from_stock(
-            gtk.ENTRY_ICON_PRIMARY, 'tryton-find')
+        self.search_entry.set_icon_from_pixbuf(
+            Gtk.EntryIconPosition.PRIMARY,
+            common.IconFactory.get_pixbuf('tryton-filter', Gtk.IconSize.MENU))
         self.search_entry.set_icon_tooltip_text(
-            gtk.ENTRY_ICON_PRIMARY, _('Open filters'))
-        self.completion = gtk.EntryCompletion()
-        self.completion.set_model(gtk.ListStore(str))
+            Gtk.EntryIconPosition.PRIMARY, _('Open filters'))
+        self.completion = Gtk.EntryCompletion()
+        self.completion.set_model(Gtk.ListStore(str))
         self.completion.set_text_column(0)
         self.completion.props.inline_selection = True
         self.completion.props.popup_set_width = False
@@ -167,7 +226,7 @@ class ScreenContainer(object):
         self.search_entry.connect('focus-in-event', self.focus_in)
         self.search_entry.connect('icon-press', self.icon_press)
 
-        hbox.pack_start(self.search_entry, expand=True, fill=True)
+        hbox.pack_start(self.search_entry, expand=True, fill=True, padding=0)
 
         def popup(widget):
             menu = widget._menu
@@ -179,76 +238,76 @@ class ScreenContainer(object):
 
             def menu_position(menu, data=None):
                 widget_allocation = widget.get_allocation()
-                if hasattr(widget.window, 'get_root_coords'):
-                    x, y = widget.window.get_root_coords(
-                        widget_allocation.x, widget_allocation.y)
-                else:
-                    x, y = widget.window.get_origin()
-                    x += widget_allocation.x
-                    y += widget_allocation.y
+                x, y = widget.get_window().get_root_coords(
+                    widget_allocation.x, widget_allocation.y)
                 return (x, y + widget_allocation.height, False)
 
             for id_, name, domain in self.bookmarks():
-                menuitem = gtk.MenuItem(name)
+                menuitem = Gtk.MenuItem(label=name)
                 menuitem.connect('activate', self.bookmark_activate, domain)
                 menu.add(menuitem)
 
             menu.show_all()
-            menu.popup(None, None, menu_position, 0, 0)
+            if hasattr(menu, 'popup_at_widget'):
+                menu.popup_at_widget(
+                    widget, Gdk.Gravity.SOUTH_WEST, Gdk.Gravity.NORTH_WEST,
+                    Gtk.get_current_event())
+            else:
+                menu.popup(None, None, menu_position, 0, 0)
 
         def deactivate(menuitem, togglebutton):
             togglebutton.props.active = False
 
-        but_bookmark = gtk.ToggleButton()
+        but_bookmark = Gtk.ToggleButton()
         self.but_bookmark = but_bookmark
         tooltips.set_tip(but_bookmark, _('Show bookmarks of filters'))
-        img_bookmark = gtk.Image()
-        img_bookmark.set_from_stock('tryton-bookmark',
-            gtk.ICON_SIZE_SMALL_TOOLBAR)
-        img_bookmark.set_alignment(0.5, 0.5)
-        but_bookmark.add(img_bookmark)
-        but_bookmark.set_relief(gtk.RELIEF_NONE)
-        menu = gtk.Menu()
+        but_bookmark.add(common.IconFactory.get_image(
+                'tryton-bookmarks', Gtk.IconSize.SMALL_TOOLBAR))
+        but_bookmark.set_relief(Gtk.ReliefStyle.NONE)
+        menu = Gtk.Menu()
         menu.set_property('reserve-toggle-size', False)
         menu.connect('deactivate', deactivate, but_bookmark)
         but_bookmark._menu = menu
         but_bookmark.connect('toggled', popup)
-        hbox.pack_start(but_bookmark, expand=False, fill=False)
+        hbox.pack_start(but_bookmark, expand=False, fill=False, padding=0)
 
-        but_prev = gtk.Button()
+        but_active = Gtk.ToggleButton()
+        self.but_active = but_active
+        self._set_active_tooltip()
+        but_active.add(common.IconFactory.get_image(
+                'tryton-archive', Gtk.IconSize.SMALL_TOOLBAR))
+        but_active.set_relief(Gtk.ReliefStyle.NONE)
+        but_active.connect('toggled', self.search_active)
+        hbox.pack_start(but_active, expand=False, fill=False, padding=0)
+
+        but_prev = Gtk.Button()
         self.but_prev = but_prev
         tooltips.set_tip(but_prev, _('Previous'))
         but_prev.connect('clicked', self.search_prev)
-        img_prev = gtk.Image()
-        img_prev.set_from_stock('tryton-go-previous',
-                gtk.ICON_SIZE_SMALL_TOOLBAR)
-        img_prev.set_alignment(0.5, 0.5)
-        but_prev.add(img_prev)
-        but_prev.set_relief(gtk.RELIEF_NONE)
-        hbox.pack_start(but_prev, expand=False, fill=False)
+        but_prev.add(common.IconFactory.get_image(
+                'tryton-back', Gtk.IconSize.SMALL_TOOLBAR))
+        but_prev.set_relief(Gtk.ReliefStyle.NONE)
+        hbox.pack_start(but_prev, expand=False, fill=False, padding=0)
 
-        but_next = gtk.Button()
+        but_next = Gtk.Button()
         self.but_next = but_next
         tooltips.set_tip(but_next, _('Next'))
         but_next.connect('clicked', self.search_next)
-        img_next = gtk.Image()
-        img_next.set_from_stock('tryton-go-next',
-                gtk.ICON_SIZE_SMALL_TOOLBAR)
-        img_next.set_alignment(0.5, 0.5)
-        but_next.add(img_next)
-        but_next.set_relief(gtk.RELIEF_NONE)
-        hbox.pack_start(but_next, expand=False, fill=False)
+        but_next.add(common.IconFactory.get_image(
+                'tryton-forward', Gtk.IconSize.SMALL_TOOLBAR))
+        but_next.set_relief(Gtk.ReliefStyle.NONE)
+        hbox.pack_start(but_next, expand=False, fill=False, padding=0)
 
         hbox.show_all()
-        hbox.set_focus_chain([self.search_entry])
-        self.filter_vbox.pack_start(hbox, expand=False, fill=False)
+        self.filter_vbox.pack_start(hbox, expand=False, fill=False, padding=0)
 
-        hseparator = gtk.HSeparator()
+        hseparator = Gtk.HSeparator()
         hseparator.show()
-        self.filter_vbox.pack_start(hseparator, expand=False, fill=False)
+        self.filter_vbox.pack_start(
+            hseparator, expand=False, fill=False, padding=0)
 
         if self.tab_domain:
-            self.notebook = gtk.Notebook()
+            self.notebook = Gtk.Notebook()
             try:
                 self.notebook.props.homogeneous = True
             except AttributeError:
@@ -256,40 +315,40 @@ class ScreenContainer(object):
                 pass
             self.notebook.set_scrollable(True)
             for name, domain, count in self.tab_domain:
-                hbox = gtk.HBox(spacing=3)
-                label = gtk.Label('_' + name)
+                hbox = Gtk.HBox(spacing=3)
+                label = Gtk.Label(label='_' + name)
                 label.set_use_underline(True)
-                hbox.pack_start(label, expand=True, fill=True)
-                counter = gtk.Label()
-                hbox.pack_start(counter, expand=False, fill=True)
+                hbox.pack_start(label, expand=True, fill=True, padding=0)
+                counter = Gtk.Label()
+                hbox.pack_start(counter, expand=False, fill=True, padding=0)
                 hbox.show_all()
-                self.notebook.append_page(gtk.VBox(), hbox)
+                self.notebook.append_page(Gtk.VBox(), hbox)
                 self.tab_counter.append(counter)
-            self.filter_vbox.pack_start(self.notebook, expand=True, fill=True)
+            self.filter_vbox.pack_start(
+                self.notebook, expand=True, fill=True, padding=0)
             self.notebook.show_all()
             # Set the current page before connecting to switch-page to not
             # trigger the search a second times.
             self.notebook.set_current_page(0)
-            self.notebook.get_nth_page(0).pack_end(self.viewport)
+            self.notebook.get_nth_page(0).pack_end(
+                self.viewport, expand=True, fill=True, padding=0)
             self.notebook.connect('switch-page', self.switch_page)
             self.notebook.connect_after('switch-page', self.switch_page_after)
             filter_expand = True
         else:
             self.notebook = None
-            self.vbox.pack_end(self.viewport)
+            self.vbox.pack_end(
+                self.viewport, expand=True, fill=True, padding=0)
             filter_expand = False
 
-        self.vbox.pack_start(self.filter_vbox, expand=filter_expand, fill=True)
+        self.vbox.pack_start(
+            self.filter_vbox, expand=filter_expand, fill=True, padding=0)
 
         self.but_next.set_sensitive(False)
         self.but_prev.set_sensitive(False)
 
         tooltips.enable()
 
-    def destroy(self):
-        if self.search_window:
-            self.search_window.hide()
-
     def widget_get(self):
         return self.vbox
 
@@ -306,7 +365,8 @@ class ScreenContainer(object):
             if self.viewport in self.vbox.get_children():
                 self.vbox.remove(self.viewport)
                 self.notebook.get_nth_page(self.notebook.get_current_page()
-                    ).pack_end(self.viewport)
+                    ).pack_end(
+                        self.viewport, expand=True, fill=True, padding=0)
 
     def hide_filter(self):
         if self.filter_vbox:
@@ -316,7 +376,8 @@ class ScreenContainer(object):
             if self.viewport not in self.vbox.get_children():
                 self.notebook.get_nth_page(self.notebook.get_current_page()
                     ).remove(self.viewport)
-                self.vbox.pack_end(self.viewport)
+                self.vbox.pack_end(
+                    self.viewport, expand=True, fill=True, padding=0)
 
     def set(self, widget):
         viewport1 = self.viewport
@@ -339,7 +400,7 @@ class ScreenContainer(object):
             model.append([r.strip()])
 
     def get_text(self):
-        return self.search_entry.get_text().decode('utf-8')
+        return self.search_entry.get_text()
 
     def set_text(self, value):
         self.search_entry.set_text(value)
@@ -355,34 +416,37 @@ class ScreenContainer(object):
         self.do_search()
 
     def bookmark_match(self):
-        icon_stock = self.search_entry.get_icon_stock(gtk.ENTRY_ICON_SECONDARY)
         current_text = self.get_text()
         if current_text:
             current_domain = self.screen.domain_parser.parse(current_text)
-            self.search_entry.set_icon_activatable(gtk.ENTRY_ICON_SECONDARY,
-                bool(current_text))
-            self.search_entry.set_icon_sensitive(gtk.ENTRY_ICON_SECONDARY,
-                bool(current_text))
+            self.search_entry.set_icon_activatable(
+                Gtk.EntryIconPosition.SECONDARY, bool(current_text))
+            self.search_entry.set_icon_sensitive(
+                Gtk.EntryIconPosition.SECONDARY, bool(current_text))
             for id_, name, domain in self.bookmarks():
                 text = self.screen.domain_parser.string(domain)
-                domain = self.screen.domain_parser.parse(text.decode('utf-8'))
+                domain = self.screen.domain_parser.parse(text)
                 if (text == current_text
                         or domain == current_domain):
-                    if icon_stock != 'tryton-star':
-                        self.search_entry.set_icon_from_stock(
-                            gtk.ENTRY_ICON_SECONDARY, 'tryton-star')
+                    self.search_entry.set_icon_from_pixbuf(
+                        Gtk.EntryIconPosition.SECONDARY,
+                        common.IconFactory.get_pixbuf(
+                            'tryton-bookmark', Gtk.IconSize.MENU))
                     self.search_entry.set_icon_tooltip_text(
-                        gtk.ENTRY_ICON_SECONDARY, _('Remove this bookmark'))
+                        Gtk.EntryIconPosition.SECONDARY,
+                        _('Remove this bookmark'))
                     return id_
-        if icon_stock != 'tryton-unstar':
-            self.search_entry.set_icon_from_stock(gtk.ENTRY_ICON_SECONDARY,
-                'tryton-unstar')
+        self.search_entry.set_icon_from_pixbuf(
+            Gtk.EntryIconPosition.SECONDARY,
+            common.IconFactory.get_pixbuf(
+                'tryton-bookmark-border', Gtk.IconSize.MENU))
         if current_text:
-            self.search_entry.set_icon_tooltip_text(gtk.ENTRY_ICON_SECONDARY,
-                _('Bookmark this filter'))
-        elif self.search_entry.get_icon_tooltip_text(gtk.ENTRY_ICON_SECONDARY):
-            self.search_entry.set_icon_tooltip_text(gtk.ENTRY_ICON_SECONDARY,
-                None)
+            self.search_entry.set_icon_tooltip_text(
+                Gtk.EntryIconPosition.SECONDARY, _('Bookmark this filter'))
+        elif self.search_entry.get_icon_tooltip_text(
+                Gtk.EntryIconPosition.SECONDARY):
+            self.search_entry.set_icon_tooltip_text(
+                Gtk.EntryIconPosition.SECONDARY, None)
 
     def search_next(self, widget=None):
         self.screen.search_next(self.get_text())
@@ -390,12 +454,24 @@ class ScreenContainer(object):
     def search_prev(self, widget=None):
         self.screen.search_prev(self.get_text())
 
+    def search_active(self, widget=None):
+        self._set_active_tooltip()
+        self.screen.search_filter(self.get_text())
+
+    def _set_active_tooltip(self):
+        if self.but_active.get_active():
+            tooltip = _('Show active records')
+        else:
+            tooltip = _('Show inactive records')
+        tooltips = common.Tooltips()
+        tooltips.set_tip(self.but_active, tooltip)
+
     def switch_page(self, notebook, page, page_num):
         current_page = notebook.get_nth_page(notebook.get_current_page())
         current_page.remove(self.viewport)
 
         new_page = notebook.get_nth_page(page_num)
-        new_page.pack_end(self.viewport)
+        new_page.pack_end(self.viewport, expand=True, fill=True, padding=0)
 
     def switch_page_after(self, notebook, page, page_num):
         self.do_search()
@@ -436,7 +512,7 @@ class ScreenContainer(object):
                 return
             self.update()
             self.search_entry.emit('changed')
-        gobject.idle_add(callback)
+        GLib.idle_add(callback)
 
     def activate(self, widget):
         if not self.search_entry.get_selection_bounds():
@@ -455,15 +531,15 @@ class ScreenContainer(object):
                 return
             self.update()
             self.bookmark_match()
-        gobject.idle_add(keypress)
+        GLib.idle_add(keypress)
 
     def icon_press(self, widget, icon_pos, event):
-        if icon_pos == gtk.ENTRY_ICON_PRIMARY:
+        if icon_pos == Gtk.EntryIconPosition.PRIMARY:
             self.search_box(widget)
-        elif icon_pos == gtk.ENTRY_ICON_SECONDARY:
-            icon_stock = self.search_entry.get_icon_stock(icon_pos)
+        elif icon_pos == Gtk.EntryIconPosition.SECONDARY:
             model_name = self.screen.model_name
-            if icon_stock == 'tryton-unstar':
+            id_ = self.bookmark_match()
+            if not id_:
                 text = self.get_text()
                 if not text:
                     return
@@ -473,8 +549,7 @@ class ScreenContainer(object):
                 domain = self.screen.domain_parser.parse(text)
                 common.VIEW_SEARCH.add(model_name, name, domain)
                 self.set_text(self.screen.domain_parser.string(domain))
-            elif icon_stock == 'tryton-star':
-                id_ = self.bookmark_match()
+            else:
                 common.VIEW_SEARCH.remove(model_name, id_)
             # Refresh icon and bookmark button
             self.bookmark_match()
@@ -489,19 +564,13 @@ class ScreenContainer(object):
             window.hide()
             self.search_entry.grab_focus()
 
-        def key_press(widget, event):
-            if event.keyval == gtk.keysyms.Escape:
-                window_hide(widget)
-                return True
-            return False
-
         def search():
-            self.search_window.hide()
+            self.search_popover.popdown()
             text = ''
-            for label, entry in self.search_table.fields:
-                if isinstance(entry, gtk.ComboBox):
+            for label, entry in self.search_grid.fields:
+                if isinstance(entry, Gtk.ComboBoxText):
                     value = quote(entry.get_active_text()) or None
-                elif isinstance(entry, (Dates, Selection)):
+                elif isinstance(entry, (Between, Selection)):
                     value = entry.get_value()
                 else:
                     value = quote(entry.get_text()) or None
@@ -513,47 +582,24 @@ class ScreenContainer(object):
             # because domain parser could simplify the text
             self.last_search_text = self.get_text()
 
-        if not self.search_window:
-            self.search_window = gtk.Window()
-            self.search_window.set_transient_for(widget.get_toplevel())
-            self.search_window.set_type_hint(
-                gtk.gdk.WINDOW_TYPE_HINT_POPUP_MENU)
-            self.search_window.set_destroy_with_parent(True)
-            self.search_window.set_decorated(False)
-            self.search_window.set_deletable(False)
-            self.search_window.connect('delete-event', window_hide)
-            self.search_window.connect('key-press-event', key_press)
-            self.search_window.connect('focus-out-event', window_hide)
+        if not self.search_popover:
+            self.search_popover = Gtk.Popover()
+            self.search_popover.set_relative_to(widget)
 
-            def toggle_window_hide(combobox, shown):
-                if combobox.props.popup_shown:
-                    self.search_window.handler_block_by_func(window_hide)
-                else:
-                    self.search_window.handler_unblock_by_func(window_hide)
-
-            vbox = gtk.VBox()
-            fields = [f for f in self.screen.domain_parser.fields.itervalues()
+            vbox = Gtk.VBox()
+            fields = [f for f in self.screen.domain_parser.fields.values()
                 if f.get('searchable', True)]
-            self.search_table = gtk.Table(rows=len(fields), columns=2)
-            self.search_table.set_homogeneous(False)
-            self.search_table.set_border_width(5)
-            self.search_table.set_row_spacings(2)
-            self.search_table.set_col_spacings(2)
+            self.search_grid = Gtk.Grid(column_spacing=3, row_spacing=3)
 
             # Fill table with fields
-            self.search_table.fields = []
+            self.search_grid.fields = []
             for i, field in enumerate(fields):
-                label = gtk.Label(field['string'])
-                label.set_alignment(0.0, 0.0)
-                self.search_table.attach(label, 0, 1, i, i + 1,
-                    yoptions=gtk.FILL)
-                yoptions = False
+                label = Gtk.Label(
+                    label=field['string'],
+                    halign=Gtk.Align.START, valign=Gtk.Align.START)
+                self.search_grid.attach(label, 0, i, 1, 1)
                 if field['type'] == 'boolean':
-                    if hasattr(gtk, 'ComboBoxText'):
-                        entry = gtk.ComboBoxText()
-                    else:
-                        entry = gtk.combo_box_new_text()
-                    entry.connect('notify::popup-shown', toggle_window_hide)
+                    entry = Gtk.ComboBoxText()
                     entry.append_text('')
                     selections = (_('True'), _('False'))
                     for selection in selections:
@@ -561,9 +607,10 @@ class ScreenContainer(object):
                 elif field['type'] == 'selection':
                     selections = tuple(x[1] for x in field['selection'])
                     entry = Selection(selections)
-                    yoptions = gtk.FILL | gtk.EXPAND
+                    entry.set_vexpand(True)
                 elif field['type'] in ('date', 'datetime', 'time'):
-                    date_format = self.screen.context.get('date_format', '%x')
+                    date_format = common.date_format(
+                        self.screen.context.get('date_format'))
                     if field['type'] == 'date':
                         entry = Dates(date_format)
                     elif field['type'] in ('datetime', 'time'):
@@ -572,73 +619,52 @@ class ScreenContainer(object):
                             entry = Times(time_format)
                         elif field['type'] == 'datetime':
                             entry = DateTimes(date_format, time_format)
-                    entry.connect_activate(lambda *a: search())
+                    entry.connect('activate', lambda *a: search())
+                elif field['type'] in ['integer', 'float', 'numeric']:
+                    entry = Numbers()
+                    entry.connect('activate', lambda *a: search())
                 else:
-                    entry = gtk.Entry()
+                    entry = Gtk.Entry()
                     entry.connect('activate', lambda *a: search())
                 label.set_mnemonic_widget(entry)
-                self.search_table.attach(entry, 1, 2, i, i + 1,
-                    yoptions=yoptions)
-                self.search_table.fields.append((field['string'], entry))
+                self.search_grid.attach(entry, 1, i, 1, 1)
+                self.search_grid.fields.append((field['string'], entry))
 
-            scrolled = gtk.ScrolledWindow()
-            scrolled.add_with_viewport(self.search_table)
-            scrolled.set_shadow_type(gtk.SHADOW_NONE)
-            scrolled.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
-            vbox.pack_start(scrolled, expand=True, fill=True)
-            find_button = gtk.Button(_('Find'))
+            scrolled = Gtk.ScrolledWindow()
+            scrolled.add(self.search_grid)
+            scrolled.set_shadow_type(Gtk.ShadowType.NONE)
+            scrolled.set_policy(
+                Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
+            vbox.pack_start(scrolled, expand=True, fill=True, padding=0)
+            find_button = Gtk.Button(label=_('Find'))
             find_button.connect('clicked', lambda *a: search())
-            find_img = gtk.Image()
-            find_img.set_from_stock('tryton-find', gtk.ICON_SIZE_SMALL_TOOLBAR)
-            find_button.set_image(find_img)
-            hbuttonbox = gtk.HButtonBox()
-            hbuttonbox.set_spacing(5)
-            hbuttonbox.pack_start(find_button)
-            hbuttonbox.set_layout(gtk.BUTTONBOX_END)
-            vbox.pack_start(hbuttonbox, expand=False, fill=True)
-            self.search_window.add(vbox)
+            find_button.set_image(common.IconFactory.get_image(
+                    'tryton-search', Gtk.IconSize.SMALL_TOOLBAR))
+            find_button.set_can_default(True)
+            self.search_popover.set_default_widget(find_button)
+            hbuttonbox = Gtk.HButtonBox()
+            hbuttonbox.pack_start(
+                find_button, expand=False, fill=False, padding=0)
+            hbuttonbox.set_layout(Gtk.ButtonBoxStyle.END)
+            vbox.pack_start(hbuttonbox, expand=False, fill=True, padding=0)
+            self.search_popover.add(vbox)
             vbox.show_all()
+            scrolled.set_size_request(
+                -1, min(self.search_grid.get_preferred_height()[1], 400))
 
-            new_size = map(sum, zip(self.search_table.size_request(),
-                    scrolled.size_request()))
-            self.search_window.set_default_size(*new_size)
-
-        parent = widget.get_toplevel()
-        widget_x, widget_y = widget.translate_coordinates(parent, 0, 0)
-        widget_allocation = widget.get_allocation()
-
-        # Resize the window to not be out of the parent
-        width, height = self.search_window.get_default_size()
-        allocation = parent.get_allocation()
-        delta_width = allocation.width - (widget_x + width)
-        delta_height = allocation.height - (widget_y + widget_allocation.height
-            + height)
-        if delta_width < 0:
-            width += delta_width
-        if delta_height < 0:
-            height += delta_height
-        self.search_window.resize(width, height)
-
-        # Move the window under the button
-        if hasattr(widget.window, 'get_root_coords'):
-            x, y = widget.window.get_root_coords(
-                widget_allocation.x, widget_allocation.y)
-        else:
-            x, y = widget.window.get_origin()
-        self.search_window.move(
-            x, y + widget_allocation.height)
-        self.search_window.show()
-        self.search_window.grab_focus()
+        self.search_popover.set_pointing_to(
+            widget.get_icon_area(Gtk.EntryIconPosition.PRIMARY))
+        self.search_popover.popup()
+        if self.search_grid.fields:
+            self.search_grid.fields[0][1].grab_focus()
 
         if self.last_search_text.strip() != self.get_text().strip():
-            for label, entry in self.search_table.fields:
-                if isinstance(entry, gtk.ComboBox):
+            for label, entry in self.search_grid.fields:
+                if isinstance(entry, Gtk.ComboBoxText):
                     entry.set_active(-1)
-                elif isinstance(entry, Dates):
-                    entry.set_values(None, None)
+                elif isinstance(entry, Between):
+                    entry.set_value(None, None)
                 elif isinstance(entry, Selection):
                     entry.treeview.get_selection().unselect_all()
                 else:
                     entry.set_text('')
-            if self.search_table.fields:
-                self.search_table.fields[0][1].grab_focus()
